﻿using UnityEngine;
using System.Collections;

public class Skeleton : MonoBehaviour {

	// the bones to create in the limbs
	public GameObject hingebone;
	// so why is this called a distbone if it uses a hinge joint? Well, the joint was originally distance, 
	// but I found using a hinge joint and odd hitboxes has a more desirable effect.
	public GameObject distbone;
	// the 4 limbs (left and right hands and feet) and their associated joints
	public Limb leftHand;
	HingeJoint2D leftHHinge;
	public Limb rightHand;
	HingeJoint2D rightHHinge;
	public Limb leftFoot;
	HingeJoint2D leftFHinge;
	public Limb rightFoot;
	HingeJoint2D rightFHinge;

	// Use this for initialization
	void Start () {
		// associate the hinge scripts with their limb
		HingeJoint2D[] hinges = gameObject.GetComponents<HingeJoint2D> ();
		foreach (HingeJoint2D hinge in hinges) 
		{
			if (hinge.connectedBody.gameObject.name == leftHand.gameObject.name) {
				leftHHinge = hinge;
			} else if (hinge.connectedBody.gameObject.name == rightHand.gameObject.name) {
				rightHHinge = hinge;
			} else if (hinge.connectedBody.gameObject.name == leftFoot.gameObject.name) {
				leftFHinge = hinge;
			} else if (hinge.connectedBody.gameObject.name == rightFoot.gameObject.name) {
				rightFHinge = hinge;
			}
		}
	}
	
	// Update is called once per frame
	void Update () {
		// check if any limbs need new bones added, then do so if they do
		if (leftHand.needsNewBone)
		{
			AddNewBone(leftHand, leftHHinge, hingebone);
		}
		if (rightHand.needsNewBone)
		{
			AddNewBone(rightHand, rightHHinge, hingebone);
		}
		if (leftFoot.needsNewBone)
		{
			AddNewBone(leftFoot, leftFHinge, distbone);
		}
		// also toggle the foot colliders if needed
		ToggleLimbColliders (leftFoot, leftFHinge);
		if (rightFoot.needsNewBone)
		{
			AddNewBone(rightFoot, rightFHinge, distbone);
		}
		ToggleLimbColliders (rightFoot, rightFHinge);
	}

	// Toggles the bones linked from the given joint to a different layer
	void ToggleLimbColliders(Limb limb, HingeJoint2D joint)
	{
		// check to see if toggling is needed because of a mismatch between state and request
		if (limb.turnOffColliders && !limb.collidersOff 
		 || !limb.turnOffColliders && limb.collidersOff)
		{
			// iterate through the joints until there are no more bones
			GameObject bone = joint.connectedBody.gameObject; 
			while (bone.GetComponent<HingeJoint2D>() != null)
			{
				if (bone.gameObject.layer == 10) { bone.gameObject.layer = 11; }
				else {bone.gameObject.layer = 10;}
				bone = bone.GetComponent<HingeJoint2D>().connectedBody.gameObject;
				print (bone);
				if (bone == null) {break;}
			}
			limb.collidersOff = !limb.collidersOff;
		}
	}

	// Add the given bone to the given limb through the given hinge
	void AddNewBone(Limb limb, HingeJoint2D hinge, GameObject bone)
	{
		limb.AddedBone(); // let the limb know a bone is being added
		Vector3 shift = new Vector3(0.28f, 0); // shift to move the bone before creating
		// make the new bone then split the current bone from the body joint and slide the new bone in
		GameObject newbone = (GameObject) Instantiate(bone, hinge.transform.position-shift, Quaternion.identity);
		Rigidbody2D nextbone = hinge.connectedBody;
		hinge.connectedBody = newbone.GetComponent<Rigidbody2D>();
		Vector2 conanchor = hinge.connectedAnchor;
		// shift the hinge locations so they all makes sense
		conanchor.x += nextbone.GetComponent<SpriteRenderer> ().bounds.size.x / 2;
		conanchor.x -= newbone.gameObject.GetComponent<SpriteRenderer>().bounds.size.x/2;
		hinge.connectedAnchor = conanchor;
		HingeJoint2D newbonehinge = newbone.GetComponent<HingeJoint2D>();
		newbonehinge.connectedBody = nextbone;
		Vector2 newboneconanchor = newbonehinge.connectedAnchor;
		newboneconanchor.x -= nextbone.gameObject.GetComponent<SpriteRenderer>().bounds.size.x/2;
		newbonehinge.connectedAnchor = newboneconanchor;
	}

	// if it's time to rattle let all the limbs know it
	public void RATTLE() 
	{
		RATTLELIMB (rightFHinge);
		RATTLELIMB (rightHHinge);
		RATTLELIMB (leftFHinge);
		RATTLELIMB (leftHHinge);
	}

	// iterate through the linked bones off a joint and start their incessant rattlin'
	void RATTLELIMB(HingeJoint2D joint)
	{
		GameObject bone = joint.connectedBody.gameObject; 
		HingeJoint2D bonehinge = bone.GetComponent<HingeJoint2D>();
		while (bonehinge != null)
		{
			bone.GetComponent<RATTLE>().rattle = true;
			bone = bone.GetComponent<HingeJoint2D>().connectedBody.gameObject;
			if (bone == null) {break;}
			bonehinge = bone.GetComponent<HingeJoint2D>();
		}
	}
}